@@ -122,6 +122,9 @@ def _maybe_create_key(key):
     if section not in data:
         data[section] = {}
 
+def iterate_sections():
+    for n in data.keys():
+        yield n
 ########################
 ## set_XXXX functions ##
 ########################
