@@ -197,20 +197,7 @@ class Gui(abstract.IntervalGui):
         # ------------------------------------------
         self._add_auto_new_question_gui(self.config_box)
         # ----------------------------------------------
-        hbox = gu.bHBox(self.config_box, False)
-        hbox.set_spacing(gnome.ui.PAD_SMALL)
-        gu.bLabel(hbox, _("Input interface:"), False)
-
-        combo = gtk.combo_box_new_text()
-        for i in range(len(inputwidgets.inputwidget_names)):
-            combo.append_text(inputwidgets.inputwidget_names[i])
-        combo.set_active(self.get_int('inputwidget'))
-        combo.connect('changed', lambda w: self.use_inputwidget(w.get_active()))
-        hbox.pack_start(combo, False)
-
-        self.g_disable_unused_buttons = gu.nCheckButton(self.m_exname,
-                    'disable_unused_intervals', _("_Disable unused buttons"))
-        hbox.pack_start(self.g_disable_unused_buttons)
+        self._create_select_inputwidget_gui()
         # ------------ frame -------------------
         self.config_box.set_spacing(0)
         self.config_box.show_all()
