@@ -18,6 +18,7 @@
 if __name__ == '__main__':
     import testsetup
 
+import glob
 import xrandom
 import uuid
 import stat
@@ -26,8 +27,7 @@ import locale
 import gethomedir
 
 class LessonfileException(Exception):
-    def get_title(self):
-        return "LessonfileException"
+    pass
 
 class NoQuestionInFileException(LessonfileException):
     """
@@ -70,9 +70,9 @@ _keys_to_interval = {
                -4: '-M3',# c -> as
                -3: 'm3', # c -> es
                -2: '-M2', # c -> bes
-               -1: '4', # c -> f
-                0: 'u',
-                1: '-4', # c -> g,
+               -1: 'p4', # c -> f
+                0: 'p1',
+                1: '-p4', # c -> g,
                 2: 'M2', # c -> d
                 3: '-m3',# c -> a
                 4: 'M3', # c -> e
@@ -84,44 +84,61 @@ _keys_to_interval = {
                 10: 'M6', # c -> ais
             }
 
-predef = {'dictation': 'dictation',
-          'rhythm': 'rhythm',
-          'harmonicprogressiondictation': 'harmonicprogressiondictation',
-          'singchord': 'singchord',
-          'singanswer': 'singanswer',
-          'chordvoicing': 'chordvoicing',
-          'chord': 'chord',
-          'compareintervals': 'compareintervals',
-          'idbyname': 'idbyname',
-          'singinterval': 'singinterval',
-          'melodicinterval': 'melodicinterval',
-          'harmonicinterval': 'harmonicinterval',
-          'idtone': 'idtone',
-          'twelvetone': 'twelvetone',
-          'identifybpm': 'identifybpm',
-          #
-          'harmonic': 'harmonic',
-          'melodic': 'melodic',
-          'progression': 'progression',
-          'normal': 'normal',
-          'voice': 'voice',
-          'rvoice': 'rvoice',
-          'satb': 'satb',
-          'cmdline': 'cmdline',
-          'wavfile': 'wavfile',
-          'horiz': 'horiz',
-          'vertic': 'vertic',
-          'yes': True,
-          'no': False,
-          'accidentals': 'accidentals',
-          'key': 'key',
-          'semitones': 'semitones',
-          'play': 'play',
-          'show': 'show',
-          'tempo': (60, 4),
-          # Used by the rhythm module:
-          'newline': 'newline',
-          }
+keywords = (
+    # exercise modules
+    'dictation',
+    'rhythm',
+    'harmonicprogressiondictation',
+    'singchord',
+    'singanswer',
+    'chordvoicing',
+    'chord',
+    'compareintervals',
+    'idbyname',
+    'singinterval',
+    'melodicinterval',
+    'harmonicinterval',
+    'idtone',
+    'twelvetone',
+    'identifybpm',
+    'nameinterval',
+    #
+    'harmonic',
+    'melodic',
+    'progression',
+    'normal',
+    'voice',
+    'rvoice',
+    'satb',
+    'cmdline',
+    'wavfile',
+    'horiz',
+    'vertic',
+    'accidentals',
+    'key',
+    'semitones',
+    'play',
+    'show',
+    # Used by the rhythm module:
+    'newline',
+    'd1', 'p1', 'a1',
+    'd2', 'm2', 'M2', 'a2',
+    'd3', 'm3', 'M3', 'a3',
+    'd4', 'p4', 'a4',
+    'd5', 'p5', 'a5',
+    'd6', 'm6', 'M6', 'a6',
+    'd7', 'm7', 'M7', 'a7',
+    'd8', 'p8', 'a8',
+    'd9', 'm9', 'M9', 'a9',
+    'd10', 'm10', 'M10', 'a10',
+)
+predef = {
+    'tempo': (60, 4),
+    'yes': True,
+    'no': False,
+}
+for n in keywords:
+    predef[n] = n
 
 lessonfile_functions = {
         '_': dataparser.dataparser_i18n_func,
@@ -208,6 +225,13 @@ class LessonfileCommon(object):
                 question['music'] = Music(question['music'], self.header.musicformat)
             question['m_lessonfile'] = self
         self.m_random = xrandom.Random(range(len(self.m_questions)))
+        if self.header.random_transpose == True:
+            self.header.random_transpose = ['key', -5, 5]
+        # Backward compatability to handle old style
+        # random_transpose = -4, 5 FIXMECOMPAT
+        if len(self.header.random_transpose) == 2:
+            self.header.random_transpose \
+                = ['semitones'] + self.header.random_transpose
     def generate_test_questions(self):
         count, t = parse_test_def(self.header.test)
         q = range(len(self.m_questions)) * count
@@ -261,7 +285,7 @@ class LessonfileCommon(object):
                 else:
                     self._idx = random.choice(available_question_idx)
             if self.header.random_transpose:
-                self.set_random_transpose()
+                self.m_transpose = self.find_random_transpose()
             if count == 10:
                 break
             if self.m_prev_question == self.get_music() \
@@ -270,24 +294,23 @@ class LessonfileCommon(object):
             break
         self.m_random.add(self._idx)
         self.m_prev_question = self.get_music()
-    def set_random_transpose(self):
-        if self.header.random_transpose == True:
-            self.handle_random_transpose_is_yes()
-        # Backward compatability to handle old style
-        # random_transpose = -4, 5 FIXMECOMPAT
-        elif len(self.header.random_transpose) == 2:
-            self.header.random_transpose \
-                = ['semitones'] + self.header.random_transpose
+    def find_random_transpose(self):
+        """
+        Return a MusicalPitch representing a suggested random
+        transposition for the currently selected question,
+        m_questions[self._idx]
+        """
         if 'key' in self.m_questions[self._idx]:
             key = self.m_questions[self._idx]['key']
         else:
             key = "c \major"
         if self.header.random_transpose[0] == 'semitones':
-            self.m_transpose = self.semitone_find_random_transpose()
+            retval = self.semitone_find_random_transpose()
             if random.randint(0, 1):
-                self.m_transpose.enharmonic_flip()
+                retval.enharmonic_flip()
         else:
-            self.m_transpose = self.find_random_transpose(key)
+            retval = self._xxx_find_random_transpose(key)
+        return retval
     def semitone_find_random_transpose(self):
         """
         Called to find random transposition in "semitone" mode.
@@ -297,7 +320,7 @@ class LessonfileCommon(object):
         return mpd.MusicalPitch().randomize(
               mpd.transpose_notename("c'", self.header.random_transpose[1]),
               mpd.transpose_notename("c'", self.header.random_transpose[2]))
-    def find_random_transpose(self, key):
+    def _xxx_find_random_transpose(self, key):
         """
         Called to create random transposition in "accidentals" or "key" mode.
         Create and return a random MusicalPitch representing this transposition.
@@ -329,8 +352,6 @@ class LessonfileCommon(object):
         interv = mpd.Interval()
         interv.set_from_string(_keys_to_interval[random.choice(range(n_down, n_up+1))])
         return mpd.MusicalPitch.new_from_notename("c'") + interv
-    def handle_random_transpose_is_yes(self):
-        self.header.random_transpose = ['key', -5, 5]
     def iterate_questions_with_unique_names(self):
         """Iterate the questions in the lessonfile, but only yield the
         first question if several questions have the same name. The
@@ -523,7 +544,7 @@ class LessonfileCommon(object):
             old = self.m_transpose
             # try really hard not to get the same tonika:
             for x in range(100):
-                self.set_random_transpose()
+                self.m_transpose = self.find_random_transpose()
                 if old != self.m_transpose:
                     break
     def is_test_complete(self):
@@ -538,10 +559,10 @@ class LessonfileCommon(object):
         if not question:
             question = self.get_question()
         if question[varname].m_musictype == 'cmdline':
-            osutils.run_external_program(str(question[varname].m_musicdata), cfg.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')), "")
+            osutils.run_external_program(str(question[varname].m_musicdata), os.path.split(self.m_t.m_lessonfile)[0], "")
         elif question[varname].m_musictype == 'wavfile':
             soundcard.play_wav_file(os.path.join(
-                self.m_t.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')),
+                os.path.split(self.m_t.m_lessonfile)[0],
                 question[varname].m_musicdata))
         elif question[varname].m_musictype in ('normal', 'satb', 'voice', 'rvoice'):
             self.play_question_musicformat_normal_satb(question, varname)
@@ -767,6 +788,43 @@ class SingChordLessonfile(LessonfileCommon):
         LessonfileCommon.__init__(self, teacher)
         self.parse_file(filename)
 
+class NameIntervalLessonfile(LessonfileCommon):
+    def __init__(self, filename, teacher):
+        LessonfileCommon.__init__(self, teacher)
+        self.parse_file(filename)
+        iquality = []
+        inumbers = []
+        self.header.intervals = [mpd.Interval(n) for n in self.header.intervals]
+        for i in self.header.intervals:
+            if i.get_quality_short() not in iquality:
+                iquality.append(i.get_quality_short())
+            if i.steps() not in inumbers:
+                inumbers.append(i.steps())
+        def quality_sort(a, b):
+            v = ['dd', 'd', 'm', 'M', 'p', 'a', 'aa']
+            return cmp(v.index(a), v.index(b))
+        iquality.sort(quality_sort)
+        inumbers.sort()
+        if not self.header.interval_number:
+            self.header.interval_number = inumbers
+        if not isinstance(self.header.interval_number, list):
+            self.header.interval_number = [self.header.interval_number]
+        if not self.header.interval_quality:
+            self.header.interval_quality = iquality
+        if not isinstance(self.header.interval_quality, list):
+            self.header.interval_number = [self.header.interval_quality]
+        if self.header.accidentals == "":
+            self.header.accidentals = 1
+        if self.header.clef == "":
+            self.header.clef = u"violin"
+        if not self.header.tones:
+            self.header.tones = [mpd.MusicalPitch.new_from_notename("b"),
+                                mpd.MusicalPitch.new_from_notename("g''")]
+        else:
+            if len(self.header.tones) != 2:
+                raise LessonfileException("Error in the lesson file '%s':\nThe length of self.header.tones is not 2" % self.m_filename)
+            self.header.tones = [mpd.MusicalPitch.new_from_notename(n) for n in self.header.tones]
+
 class IdByNameLessonfile(LessonfileCommon):
     def __init__(self, filename, teacher):
         """
@@ -854,26 +912,27 @@ class LessonFileManager:
     def __init__(self):
         self.parse()
     def parse(self):
+        lessonpath = ['lesson-files',
+                      os.path.join(gethomedir.get_home_dir(), 'lessonfiles')]
         self.m_uiddb = {}
         vim_tmpfile_re = re.compile("\..*\.swp")
-        for collection in cfg.get_list("config/lessoncollections"):
-            if not os.path.isdir(gethomedir.expanduser(cfg.get_string("lessoncollections/%s" % collection))):
+        for dir in lessonpath:
+            if not os.path.isdir(dir):
+                print "warning: invalid directory in path:", dir
                 continue
-            # filter out backup files
-            v = filter(lambda x, r=vim_tmpfile_re: x[-1:] != '~' and not r.match(x), os.listdir(gethomedir.expanduser(cfg.get_string("lessoncollections/%s" % collection))))
+            v = glob.glob(os.path.join(dir, "*"))
             for filename in v:
                 # since I usually run solfege from the source dir:
-                if filename in ('.arch-ids', 'Makefile', 'Makefile.in') \
-                  or not os.path.isfile(self.get_complete_filename(collection, filename)):
+                if os.path.split(filename)[-1] in ('.arch-ids', 'Makefile', 'Makefile.in') \
+                  or not os.path.isfile(filename):
                     continue
                 # We save the returned lesson_id because we need it below
-                lesson_id = self.parse_into_uiddb(collection, filename)
+                lesson_id = self.parse_into_uiddb(filename)
                 # We have to check if the lesson file has changed,
                 # and if it has, then the results has to be deleted.
                 # This because Solfege has no way of knowing what kind
                 # of changes has been done to the lesson file.
-                fn = self.get_complete_filename(collection, filename)
-                h = hash(open(fn, 'r').read())
+                h = hash(open(filename, 'r').read())
                 hash_filename = os.path.join(gethomedir.expanduser("~/.solfege"),
                            "testresults", "%s_hash" % lesson_id)
                 if os.path.exists(hash_filename):
@@ -883,32 +942,29 @@ class LessonFileManager:
                                 ".solfege", "testresults", lesson_id)
                         for f in os.listdir(dirname):
                             os.remove(os.path.join(dirname, f))
-    def parse_into_uiddb(self, collection, filename):
+    def parse_into_uiddb(self, filename):
                 """
                 Returns the lesson_id of the parsed file.
                 """
-                fn = self.get_complete_filename(collection, filename)
                 try:
-                    f = LessonIdParser(fn)
+                    f = LessonIdParser(filename)
                     if not f.has_lesson_id():
                         f.add_lesson_id()
                 except IOError, e:
-                    print >> sys.stderr, "add_lesson_id('%s'):" % fn, e
+                    print >> sys.stderr, "add_lesson_id('%s'):" % filename, e
                     return
                 try:
                     p = dataparser.Dataparser(predef, lessonfile_functions)
-                    p.parse_file(fn)
+                    p.parse_file(filename)
                 except dataparser.DataparserException, e:
-                    print >> sys.stderr, "LessonFileManager.parse: Discarding lessonfile '%s' because the file contain errors:" % fn
+                    print >> sys.stderr, "LessonFileManager.parse: Discarding lessonfile '%s' because the file contain errors:" % filename
                     print >> sys.stderr, str(e)
                     return
                 if not 'title' in p.header:
                     p.header['title'] = '@@@NOT TITLE'
                 n = {
-                   'filename': fn,
-                   'lessoncollection': collection,
-                   'lessonfile': filename,
-                   'mtime': os.stat(fn)[stat.ST_MTIME],
+                   'filename': filename,
+                   'mtime': os.stat(filename)[stat.ST_MTIME],
                    'header': p.header,
                 }
                 if not p.header['lesson_id'] in self.m_uiddb:
@@ -933,14 +989,9 @@ class LessonFileManager:
             self.m_htmldoc = "%s<h2>%s</h2>" % (self.m_htmldoc, module)
             self.m_htmldoc += "<ul>"
             for n in d[module]:
-                if self.get(n, 'lessoncollection') != 'solfege':
-                    collection_info = "[%s]" % self.get(n, 'lessoncollection')
-                else:
-                    collection_info = ""
-                self.m_htmldoc += "<li><a href='solfege:practise/%s'>%s%s</a>: %s</li>" % (
+                self.m_htmldoc += "<li><a href='solfege:practise/%s'>%s</a>: %s</li>" % (
                     n,
-                    collection_info,
-                    self.get(n, 'lessonfile'),
+                    self.get(n, 'filename'),
                     self.get(n, 'title'))
             self.m_htmldoc += "</ul>"
     def get(self, lesson_id, fieldname):
@@ -982,15 +1033,7 @@ class LessonFileManager:
         FIXME: the function name is not very good.
         Return data used to fix things when we have a lesson_id crash.
         """
-        return [{'filename': d['filename'], 'lessoncollection': d['lessoncollection'], 'lessonfile': d['lessonfile'], 'timestr': time.strftime('%c', time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
-    def get_complete_filename(self, catalog, fn):
-        """
-        Return the absolute path to a file in a given collection
-        """
-        if not catalog:
-            catalog = 'solfege'
-        return gethomedir.expanduser(os.path.join(
-         cfg.get_string('lessoncollections/%s' % catalog), fn))
+        return [{'filename': d['filename'], 'timestr': time.strftime('%c', time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
 
 
 class LessonIdParser(object):
