@@ -243,14 +243,7 @@ class Url:
         elif self.protocol == 'solfege':
             v = path.split("/")
             self.action = v[0]
-            if len(v) > 1:
-                self.lessoncollection = v[1]
-            else:
-                self.lessoncollection = None
-            if len(v) > 2:
-                self.lessonfile = v[2]
-            else:
-                self.lessonfile = None
+            self.lessonfile = "/".join(v[1:])
             if len(v) > 2 and self.lessonfile and '?' in self.lessonfile:
                 self.lessonfile, self.config = self.lessonfile.split('?', 1)
             else:
@@ -262,21 +255,14 @@ class TestUrlClass(unittest.TestCase):
     def setUp(self):
         pass
     def test_all(self):
-        url = Url('solfege:practise/solfege/melodic-intervals?ask_for_names=[1, 2, 3]')
+        url = Url('solfege:practise/lesson-files/melodic-intervals?ask_for_names=[1, 2, 3]')
         self.assert_(url.protocol == 'solfege')
         self.assert_(url.action == 'practise')
-        self.assert_(url.lessoncollection == 'solfege')
-        self.assert_(url.lessonfile == 'melodic-intervals')
+        self.assert_(url.lessonfile == 'lesson-files/melodic-intervals')
         self.assert_(url.config == {'ask_for_names': '[1, 2, 3]'})
-        url = Url('solfege:/solfege/melodic-intervals?ask_for_names=[1, 2, 3]')
+        url = Url('solfege://home/tom/lessonfiles/melodic-intervals?ask_for_names=[1, 2, 3]')
         self.assert_(url.protocol == 'solfege')
-        self.assert_(url.lessoncollection == 'solfege')
-        self.assert_(url.lessonfile == 'melodic-intervals')
-        self.assert_(url.config == {'ask_for_names': '[1, 2, 3]'})
-        url = Url('solfege:/solfege/melodic-intervals?ask_for_names=[1, 2, 3]')
-        self.assert_(url.protocol == 'solfege')
-        self.assert_(url.lessoncollection == 'solfege')
-        self.assert_(url.lessonfile == 'melodic-intervals')
+        self.assert_(url.lessonfile == '/home/tom/lessonfiles/melodic-intervals')
         self.assert_(url.config == {'ask_for_names': '[1, 2, 3]'})
 
 
